# This file is auto-generated by LitServe.
# Disable auto-generation by setting `generate_client_file=False` in `LitServer.run()`.

import requests

response = requests.post(
    "http://127.0.0.1:8000/chat",
    json={"query": "What is the weather in London?"},
    stream=True,
)
for line in response.iter_lines():
    if line:
        print(line, flush=True)
